package hexedit;

/**
 * A source of Hexidecimal data.
 *
 * @author     mace
 * @created    June 10, 2003
 * @modified   $Date: 2003/06/13 21:36:34 $ by $Author: bemace $
 * @version    $Revision: 1.3 $
 */
public interface HexData {

	/**
	 * Returns the number rows (including partial rows)
	 *
	 * @return   The rowCount value
	 */
	public int getRowCount();

	/**
	 * Returns the number of columns to use when displaying this data.
	 *
	 * @return   The columnCount value
	 */
	public int getColumnCount();

	/**
	 * Gets the number of bytes in the last row, since the length of the data may
	 * not be evenly divisible by the number of columns.
	 *
	 * @return   The lastRowSize value
	 */
	public int getLastRowSize();

	/**
	 * Returns the byte at the given coordinates
	 *
	 * @param row  Description of the Parameter
	 * @param col  Description of the Parameter
	 * @return     The byte value
	 */
	public byte getByte(int row, int col);

	/**
	 * Sets the byte at the given coordinates to the given value
	 *
	 * @param row    The new byte value
	 * @param col    The new byte value
	 * @param value  The new byte value
	 */
	public void setByte(int row, int col, byte value);

	/**
	 * Returns a row of data.
	 *
	 * @param row  Description of the Parameter
	 * @return     The row value
	 */
	public byte[] getRow(int row);

	/**
	 * Returns true is the data is editable.
	 *
	 * @return   The editable value
	 */
	public boolean isEditable();
}

