package hexedit;

import javax.swing.table.*;

import org.gjt.sp.jedit.*;
import org.gjt.sp.util.*;

/**
 * Allows a binary file to be displayed and edited in a table.
 *
 * @author     mace
 * @created    June 10, 2003
 * @modified   $Date: 2003/06/13 07:21:47 $ by $Author: bemace $
 * @version    $Revision: 1.3 $
 * @see        hexedit.gui.HexTable
 */
public class HexTableModel extends AbstractTableModel {
	public final static byte[] MASKED_CHARS = {
			(byte) '\t', (byte) '\r', (byte) '\n'
			};

	private HexData _data;

	public HexTableModel(HexData data) {
		_data = data;
	}

	public String getColumnName(int col) {
		if (col == 0) {
			return "Offset";
		} else if (col == getColumnCount() - 1) {
			return "Ascii representation";
		} else {
			return Integer.toHexString(col - 1).toUpperCase();
		}
	}

	public int getColumnCount() {
		return _data.getColumnCount() + 2;
	}

	public int getRowCount() {
		return _data.getRowCount();
	}

	public Object getValueAt(int row, int col) {
		if (col == 0) {
			return Integer.toHexString(row * _data.getColumnCount());
		}
		if (col == getColumnCount() - 1) {
			byte[] bytes = _data.getRow(row);
			for (int i = 0; i < bytes.length; i++) {
				for (int n = 0; n < MASKED_CHARS.length; n++) {
					if (bytes[i] == MASKED_CHARS[n]) {
						bytes[i] = (byte) '.';
					}
				}
			}
			return new String(bytes);
		}
		int dataCol = col - 1;
		if (row != _data.getRowCount() - 1 || dataCol < _data.getLastRowSize()) {
			return new Byte(_data.getByte(row, dataCol));
		}
		return null;
	}

	/**
	 * Returns true if the cell at (row,col) is editable. Hex byte columns and the
	 * ascii column are editable.
	 *
	 * @param row  Description of the Parameter
	 * @param col  Description of the Parameter
	 * @return     The cellEditable value
	 */
	public boolean isCellEditable(int row, int col) {
		if (col == 0 || !_data.isEditable()) {
			return false;
		}
		if (col == getColumnCount() - 1) {
			return true;
		}
		int dataCol = col - 1;
		if (row == _data.getRowCount() - 1) {
			return (dataCol < _data.getLastRowSize());
		} else {
			return true;
		}
	}

	/**
	 * Sets the value at <code>row</code>,<code>col</code> to <code>value</code>.
	 * For hex bytes, <code>value.toString()</code> must be parsable as a hex
	 * value.
	 *
	 * @param value  The new value
	 * @param row    the cell's row
	 * @param col    the cell's column
	 */
	public void setValueAt(Object value, int row, int col) {
		if (!isCellEditable(row, col)) {
			return;
		}
		if (col > 0 && col < getColumnCount() - 1) {
			try {
				_data.setByte(row, col - 1, Integer.valueOf(value.toString(), 16).byteValue());
				fireTableRowsUpdated(row, row);
			} catch (NumberFormatException e) {
				Object[] args = {value};
				GUIUtilities.error(jEdit.getActiveView(), "hexedit.error.bad-number", args);
			}
		}
		if (col == getColumnCount() - 1) {
			Log.log(Log.DEBUG, this, "Old value=" + getValueAt(row, col) + ", New value=" + value);
			String old = getValueAt(row, col).toString();
			String newValue = value.toString();
			for (int i = 0; i < old.length(); i++) {
				if (old.charAt(i) != newValue.charAt(i)) {
					_data.setByte(row, i, (byte) newValue.charAt(i));
				}
			}
			fireTableRowsUpdated(row, row);
		}
	}
}

