package hexedit.gui;

import java.awt.*;
import javax.swing.*;
import javax.swing.table.*;

import hexedit.*;

/**
 * Renders <code>Byte</code> values in hexidecimal. Masked characters (i.e. tab,
 * CR, LF) are colored gray.
 *
 * @author     mace
 * @created    June 13, 2003
 * @modified   $Date: 2003/06/13 07:21:48 $ by $Author: bemace $
 * @version    $Revision: 1.3 $
 */
public class HexRenderer extends DefaultTableCellRenderer {
	public Component getTableCellRendererComponent(JTable table,
			Object value,
			boolean isSelected,
			boolean hasFocus,
			int row,
			int column) {
		super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);
		setHorizontalAlignment(CENTER);
		if (value == null) {
			setValue("");
			return this;
		}

		byte b = ((Byte) value).byteValue();

		if (isMasked(b)) {
			setForeground(Color.GRAY);
		} else {
			setForeground(Color.BLACK);
		}

		int i = NumberUtilities.byteToInt(b);
		String hex = Integer.toHexString(i).toUpperCase();
		if (hex.length() == 1) {
			setValue("0" + hex);
		} else {
			setValue(hex);
		}

		return this;
	}

	private boolean isMasked(byte b) {
		byte[] masked = HexTableModel.MASKED_CHARS;
		for (int i = 0; i < masked.length; i++) {
			if (masked[i] == b) {
				return true;
			}
		}
		return false;
	}
}

