package hexedit.gui;

import java.awt.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.table.*;

import hexedit.*;

/**
 * A table for viewing and editing binary files represented in hexidecimal.
 *
 * @author     mace
 * @created    June 13, 2003
 * @modified   $Date: 2003/06/13 07:21:48 $ by $Author: bemace $
 * @version    $Revision: 1.5 $
 */
public class HexTable extends JTable {
	public HexTable(HexTableModel model) {
		super(model);
		init();
	}

	public void init() {
		Font font = new Font("courier", Font.PLAIN, 12);
		setFont(font);

		//{{{ initialize columns
		FontMetrics fm = getFontMetrics(font);
		TableColumnModel tcm = getColumnModel();
		TableColumn offset = tcm.getColumn(0);
		offset.setCellRenderer(new OffsetRenderer());
		offset.setPreferredWidth(fm.stringWidth("X") * 8);
		for (int i = 1; i < tcm.getColumnCount() - 1; i++) {
			TableColumn column = tcm.getColumn(i);
			column.setCellRenderer(new HexRenderer());
			column.setCellEditor(new HexCellEditor());
			int width = fm.stringWidth("###");
			column.setMinWidth(width);
			column.setPreferredWidth(width);
			column.setMaxWidth(width);
		}
		TableColumn ascii = tcm.getColumn(getColumnCount() - 1);
		ascii.setCellRenderer(new AsciiRenderer());
		ascii.setCellEditor(new AsciiEditor());
		ascii.setMinWidth(fm.stringWidth("X") * getColumnCount() - 2 + 4);
		ascii.setPreferredWidth(300);
		//}}}

		setDragEnabled(false);
		setShowGrid(false);
		setRowSelectionAllowed(false);
		setColumnSelectionAllowed(false);
		setCellSelectionEnabled(true);
		setSelectionMode(ListSelectionModel.SINGLE_SELECTION);

	}

	public void setModel(HexTableModel m) {
		super.setModel(m);
		init();
	}

	/**
	 * Overridden to update selection in ascii renderer as well.
	 *
	 * @param rowIndex     Description of the Parameter
	 * @param columnIndex  Description of the Parameter
	 * @param toggle       Description of the Parameter
	 * @param extend       Description of the Parameter
	 */
	public void changeSelection(int rowIndex, int columnIndex, boolean toggle, boolean extend) {
		super.changeSelection(rowIndex, columnIndex, toggle, extend);
		if (columnIndex > 0 && columnIndex < getColumnCount() - 1) {
			tableChanged(new TableModelEvent(getModel(), rowIndex, rowIndex, columnIndex - 1));
		}
	}
}

