package hexedit.io;

import hexedit.*;

/**
 * An abstract HexData source. Subclasses must implement <code>length()</code>,
 * <code>getByte()</code>, <code>setByte()</code>, and <code>getRow()</code>.
 *
 * @author     mace
 * @created    June 11, 2003
 * @modified   $Date: 2003/06/13 21:36:34 $ by $Author: bemace $
 * @version    $Revision: 1.4 $
 */
public abstract class AbstractHexData implements HexData {

	public int getColumnCount() {
		return 16;
	}

	public int getRowCount() {
		int rows = (int) length() / getColumnCount();
		if (length() % getColumnCount() != 0) {
			rows++;
		}
		return rows;
	}

	public int getLastRowSize() {
		return (int) length() % getColumnCount();
	}

	/**
	 * Returns true if this data is editable. Read-only subclasses should override
	 * this to return <code>false</code>.
	 *
	 * @return   The editable value
	 */
	public boolean isEditable() {
		return true;
	}

	public abstract long length();

	/**
	 * gets the offset corresponding to row,col
	 *
	 * @param row  Description of the Parameter
	 * @param col  Description of the Parameter
	 * @return     The offset value
	 */
	protected int getOffset(int row, int col) {
		return row * getColumnCount() + col;
	}
}

