package hexedit.io;

import org.gjt.sp.jedit.*;
import org.gjt.sp.jedit.textarea.*;

import hexedit.*;

/**
 * Allows a Buffer to be used as a HexData source.  Unfortunately, 
 * Buffers don't save binary files correctly.
 *
 * @author     mace
 * @created    June 10, 2003
 * @modified   $Date: 2003/06/12 06:04:05 $ by $Author: bemace $
 * @version    $Revision: 1.2 $
 */
public class BufferAdapter implements HexData {
	private Buffer _buffer;
	public BufferAdapter(Buffer b) { 
		_buffer = b;
	}

	public int getRowCount() { 
		int rows = _buffer.getLength() / getColumnCount();
		if (_buffer.getLength() % getColumnCount() != 0) {
			rows++;	
		}
		return rows;
	}

	public int getColumnCount() { 
		return 16;
	}

	public int getLastRowSize() { 
		return _buffer.getLength() % getColumnCount();
	}

	public byte getByte(int row, int col) { 
		int offset = row * getColumnCount() + col;
		byte[] bytes = _buffer.getText(offset,1).getBytes();
		return bytes[0];
	}

	public void setByte(int row, int col, byte value) { 
		int offset = row * getColumnCount() + col;
		JEditTextArea textArea = jEdit.getActiveView().getTextArea();
		textArea.setSelection(new Selection.Range(offset,offset+1));
		textArea.scrollToCaret(false);
		byte[] bytes = new byte[1];
		bytes[0] = value;
		textArea.setSelectedText(new String(bytes));
	}

	public byte[] getRow(int row) {
		int start = row * getColumnCount();
		
		if (row == getRowCount() - 1) {
			return _buffer.getText(start, getLastRowSize()).getBytes();
		} else {
			return _buffer.getText(start, getColumnCount()).getBytes();
		}
	}
	
	public boolean isEditable() {
		return false;
	}
	
}

