package hexedit.io;

import java.io.*;

import hexedit.util.*;

/**
 * Description of the Class
 *
 * @author     mace
 * @created    June 11, 2003
 * @modified   $Date: 2003/06/12 17:10:58 $ by $Author: bemace $
 * @version    $Revision: 1.1 $
 */
public class BufferedHexFile extends AbstractHexData {
	private ByteBuffer _buffer = new ByteBuffer();
	private File _file;

	public final static int FILE_NOT_CHANGED = 1;
	public final static int FILE_CHANGED = 2;
	public final static int FILE_DELETED = 3;

	private int _status;

	public BufferedHexFile(File file) {
		_file = file;
	}

	public byte getByte(int row, int col) {
		return _buffer.getByte(getOffset(row, col));
	}

	public void setByte(int row, int col, byte value) {
		_buffer.setByte(getOffset(row, col), value);
		_status = FILE_CHANGED;
	}

	public byte[] getRow(int row) {
		byte[] bytes;
		if (row == getRowCount() - 1) {
			bytes = new byte[getLastRowSize()];
		} else {
			bytes = new byte[getColumnCount()];
		}
		_buffer.readBytes(row * getColumnCount(), bytes);
		return bytes;
	}

	public void load() throws IOException {
		FileInputStream in = new FileInputStream(_file);
		byte[] bytes = new byte[(int) _file.length()];
		in.read(bytes);
		in.close();
		_buffer.load(bytes);
		_status = FILE_NOT_CHANGED;
	}

	public void save() throws IOException {
		FileOutputStream out = new FileOutputStream(_file);
		out.write(_buffer.getBytes());
		out.close();
		_status = FILE_NOT_CHANGED;
	}

	public int getStatus() {
		return _status;
	}

	public long length() {
		return _buffer.size();
	}
}

