package hexedit.io;

import java.io.*;

import hexedit.*;

/**
 * Allows a File to be used as a HexData source. Changes will be written to disk
 * IMMEDIATELY.
 *
 * @author     mace
 * @created    June 10, 2003
 * @modified   $Date: 2003/06/12 06:47:38 $ by $Author: bemace $
 * @version    $Revision: 1.3 $
 */
public class HexFile extends RandomAccessFile implements HexData {
	public HexFile(File f) throws FileNotFoundException {
		super(f, "rws");
	}

	//{{{ HexData implementation
	public int getRowCount() {
		int rows = (int) length() / getColumnCount();
		if (length() % getColumnCount() != 0) {
			rows++;
		}
		return rows;
	}

	public int getColumnCount() {
		return 16;
	}

	public int getLastRowSize() {
		return (int) length() % getColumnCount();
	}

	public byte getByte(int row, int col) {
		try {
			seek(row * getColumnCount() + col);
			return readByte();
		} catch (IOException e) {
			System.out.println(e);
			return (byte) 0;
		}
	}

	public void setByte(int row, int col, byte value) {
		try {
			seek(row * getColumnCount() + col);
			writeByte(value);
		} catch (IOException e) {
			System.out.println(e);
		}
	}

	public byte[] getRow(int row) {
		try {
			byte[] bytes;
			if (row == getRowCount() - 1) {
				bytes = new byte[getLastRowSize()];
			} else {
				bytes = new byte[getColumnCount()];
			}
			seek(row * getColumnCount());
			read(bytes);
			return bytes;
		} catch (IOException e) {
			System.out.println(e);
			return new byte[getColumnCount()];
		}
	}

	public boolean isEditable() {
		return true;
	}

	public long length() {
		try {
			return super.length();
		} catch (IOException e) {
			System.out.println(e);
			return 0;
		}

	}
	//}}}

}

